/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        03.11.2007
 Description:    Webserver Config-File

 Modifiziert fr AVR-NET-IO von Pollin durch RoBue u.a.
 
 Grundidee: Konfiguration ber EINE Datei (= config.h)
 
 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#ifndef _CONFIG_H_
	#define _CONFIG_H_	
	
	//Version
	#define SoftVers "V1.0.99"
	#define RoBueVers "Version 1.3 RoBue vom 15.12.2008"
	

// Konfiguration des Systems
// -------------------------

	//Taktfrequenz (bei AVR-NET-IO i.d.R. 16MHz
	#define F_CPU 16000000UL	
	//#define F_CPU 14745600UL
	//#define F_CPU 11059200UL
	
	//Timertakt intern oder extern
	#define EXTCLOCK 0 //0=Intern 1=Externer Uhrenquarz

	//Baudrate der seriellen Schnittstelle
	#define BAUDRATE 9600
	// define BAUDRATE 19200 // -> Handy Siemens C35i
	

// PORTS (HEX) (1=OUTPUT / 0=INPUT)
// --------------------------------

	// PORTA: 0-3 -> dig. input, 4-6 -> analog. input, 7 - 1-Wire
	// (PORTA4-7 ist mit blauen Schraubklemmen verbunden)
	#define OUTA 		0x00

	//PORTB -> ENC-Netzwerkkontroller
	//nur ndern wenn man wei was man macht!

	// PORTC: -> LCD oder Schaltausgnge
	// Bei LCD mit "//" auskommentieren
	//#define OUT 		0xFF

	// PoRTD: 0,1 -> UART, 2-7 Schaltausgnge
	#define OUTD 		0xFC

	//AD-Wandler benutzen?
	#define USE_ADC		1

	// Zuordnung der Aufgaben von PORTD2-7 -> webpage.h
	#define Ausg_D2	"D2_Heizung_1"
	#define Ausg_D3	"D3_Heizung_2"
	#define Ausg_D4	"D4_Licht_1"
	#define Ausg_D5	"D5_Licht_2"
	#define Ausg_D6	"D6_Licht_3"
	#define Ausg_D7	"D7_Licht_4"


// LCD Einstellungen
// -----------------

	//LCD ja/nein
	#define USE_SER_LCD	1
	//LCD im 4Bit Mode oder seriell
	#define USE_LCD_4Bit	1
	// LCD im 4-Bit: 1 1; LCD seriell: 1 0 


	//Anzahl der Zeilen 1,2 oder 4
	//#define ONE_LINES					
	//#define TWO_LINES					
	//#define THREE_LINES					
	#define FOUR_LINES					

	#if USE_LCD_4Bit
	//LCD_D0 - LCD_D3 connect to GND
	//Im 4Bit Mode LCD_D4-->PORTx.0 ........ LCD_D7-->PORTx.3
	//LCD_RS --> PORTx.4
	//LCD_RW --> PORTx.5
	//LCD_E --> PORTx.6
	//PORTx.7-->NotConnect oder Backlight

		//Port an dem das Display angeschlossen wurde:
		#define LCD_Port_DDR		DDRC
		#define LCD_Port_Write		PORTC
		#define LCD_Port_Read		PINC

		#define LCD_RS			4 
		#define LCD_RW			5
		#define LCD_E			6

		#define LCD_DataOutput		0x0f
		#define LCD_DataInput		0x00

		#define BUSYBIT			7
	#else	
		#define PORT_LCD_DATA_ENABLE	PORTC
		#define DDR_LCD_DATA_ENABLE	DDRC
		#define LCD_DATA_ENABLE		2
	
		#define PORT_LCD_CLOCK		PORTC
		#define DDR_LCD_CLOCK		DDRC
		#define LCD_CLOCK		4
	
		#define PORT_LCD_DATA		PORTC
		#define DDR_LCD_DATA		DDRC
		#define LCD_DATA		3
	

	
		#define LCD_RS_PIN 		0
		#define LCD_LIGHT_PIN		7

	#endif //USE_LCD_4Bit
	

// Netzwerkeinstellungen
// ---------------------

	//ETH_M32_EX (www.ulrichradig.de)
	#define USE_ENC28J60	1
	//Holger Buss (www.mikrocontroller.com) Mega32-Board
	#define USE_RTL8019		0

	//IP des Webservers
	#define MYIP		IP(192,168,0,99)

	//Netzwerkmaske
	#define NETMASK		IP(255,255,255,0)

	//IP des Routers
	#define ROUTER_IP	IP(192,168,0,3)

	//IP des NTP-Servers z.B. Server 1.de.pool.ntp.org
	#define USE_NTP		1 // 1= NTP Client on
	#define NTP_IP		IP(77,37,6,59)

	//Broadcast-Adresse fr WOL
	#define USE_WOL		0 //1 = WOL on
	#define WOL_BCAST_IP	IP(192,168,0,255)
	#define WOL_MAC 	{0x00,0x1A,0xA0,0x9C,0xC6,0x0A}
	
	//MAC Adresse des Webservers	
	#define MYMAC1	0x00
	#define MYMAC2	0x22
	#define MYMAC3	0xF9
	#define MYMAC4	0x01
	#define MYMAC5	0x00
	#define MYMAC6	0x01

	//Umrechnung von IP zu unsigned long
	#define IP(a,b,c,d) ((unsigned long)(d)<<24)+((unsigned long)(c)<<16)+((unsigned long)(b)<<8)+a

	//Webserver mit Passwort? (0 == mit Passwort)
	#define HTTP_AUTH_DEFAULT	1
	
	//AUTH String "USERNAME:PASSWORT" max 14Zeichen 
	//fr Username:Passwort
	#define HTTP_AUTH_STRING "admin:uli1"


// 1-Wire-Einstellungen
// --------------------

	// 1-Wire ja/nein (0=nein, 1=ja)
	#define USE_OW		1
	// max. Anzahl der Sensoren
	#define MAXSENSORS	4
	// z.Z. nur EIN 1-Wire-Bus
	#define OW_ONE_BUS	1
	// Lnge der ID (rom-code) des Sensors incl. CRC
	#define OW_ROMCODE_SIZE 8

	// Port fuer 1-Wire
	#define OW_PIN  PA7
	#define OW_IN   PINA
	#define OW_OUT  PORTA
	#define OW_DDR  DDRA

	// ROM-IDs der DS1820 eintragen
	#define OW_ID_T01	{0x10,0x7a,0x8c,0x48,0x01,0x08,0x00,0x67}
	#define OW_ID_T02	{0x10,0x44,0x9e,0x53,0x01,0x08,0x00,0xca}
//	#define OW_ID_T03	{0x28,0x11,0x22,0x33,0x44,0,0,0x77}
//	#define OW_ID_T04	{0x28,0x22,0x33,0x44,0x55,0,0,0x77}
	#define OW_ID_Last	{0x00,0x00,0x00,0x00,0x00,0,0,0x00}

	// Entsprechende Ortszuweisung fr die Sensoren
	// -> webpage.h sieht dann so aus (Ausgabe Sensor 0 = T01):
	// "<td><p align=\"center\">Sensor 0</p></td>"
	// "<td><p align=\"center\">"
	// T01
	// "</p></td>"
	// "<td><p align=\"center\">%OW@00</p></td>"
	// "<td><p align=\"center\">Grad C</p></td>"
	// "</tr>\r\n"
	#define T01	"Abstellkammer"
	#define T02	"Computerzimmer"
	#define T03	"(nirgendwo)"
	#define T04	"(anderswo)"

	
// Kamera mit einbinden
// --------------------
//Achtung: Kamera arbeitet nur mit einem 14,7456Mhz Quarz!

	#define USE_CAM			0
	#define USE_SERVO		0
	//In cam.c knnen weitere Parameter eingestellt werde
	//z.B. Licht, Kompression usw.
	//Auflsungen
	//0 = 160x120 Pixel krzer (zum testen OK ;-)
	//1 = 320x240 Pixel ca. 10 Sek. bei einem Mega644
	//2 = 640x480 Pixel lnger (dauert zu lang!)
	#define CAM_RESELUTION	0
		

// Rest
// ----	

	//Emailversand benutzen? Konfiguration des
	//Emailclient in der Sendmail.h
	#define USE_MAIL        0
    
	//Empfang von Wetterdaten auf der Console (ber HTTP_GET)
	#define GET_WEATHER     0
    
	//Commandos und Ausgaben erfolgen ber Telnet
	#define CMD_TELNET      0

	#define MAX_VAR_ARRAY	20	//    ursprnglich 10
    
#endif //_CONFIG_H